#!/usr/bin/env python
# -*- coding: utf-8 -*-
import email
from email.header import decode_header
from email.parser import Parser as EmailParser
from email.utils import parseaddr
from imaplib import IMAP4
from io import StringIO


class NotSupportedMailFormat(Exception):
    pass


def parse_attachment(message_part):
    """
    parse attachement file
    """
    content_disposition = message_part.get("Content-Disposition", None)
    if content_disposition:
        dispositions = content_disposition.strip().split(";")
        if bool(content_disposition and dispositions[0].lower() == "attachment"):

            file_data = message_part.get_payload(decode=True)
            attachment = StringIO(file_data)
            attachment.data = file_data
            attachment.content_type = message_part.get_content_type()
            attachment.size = len(file_data)
            attachment.name = None
            attachment.create_date = None
            attachment.mod_date = None
            attachment.read_date = None

            for param in dispositions[1:]:
                name, value = param.split("=")
                if value[0] == '"':
                    value = value[1:]
                if value[-1] == '"':
                    value = value[:-1]
                if value[0] == "'":
                    value = value[1:]
                if value[-1] == "'":
                    value = value[:-1]
                name = name.lower()
                if "filename" in name:
                    attachment.name = value
                elif "create-date" in name:
                    attachment.create_date = value
                elif "modification-date" in name:
                    attachment.mod_date = value
                elif "read-date" in name:
                    attachment.read_date = value
            return attachment

    return None


def parse_email_content(content):
    """
    parse email content
    """
    p = EmailParser()
    msgobj = p.parse(StringIO(content), False)
    if msgobj['Subject'] is not None:
        decodefrag = decode_header(msgobj['Subject'])
        subj_fragments = []
        for s, enc in decodefrag:
            if enc:
                s = s.encode('utf8', 'replace')
            subj_fragments.append(s)
        subject = ''.join(subj_fragments)
    else:
        subject = None

    attachments = []
    text = None
    body = None
    html = None
    for part in msgobj.walk():
        charset = part.get_content_charset()
        if not charset:
            charset = 'utf-8'
        attachment = parse_attachment(part)
        if attachment:
            attachments.append(attachment)
        elif part.get_content_type() == "text/plain":
            if body is None:
                body = ""
            if text is None:
                text = ""
            text += str(
                part.get_payload(decode=True),
                charset,
                'replace'
            )
            body += str(
                part.get_payload(decode=True),
                charset,
                'replace'
            ).encode('utf8', 'replace')
        elif part.get_content_type() == "text/html":
            if html is None:
                html = ""
            html += str(
                part.get_payload(decode=True),
                charset,
                'replace'
            ).encode('utf8', 'replace')
    return {
        'subject':     subject,
        'body':        body,
        'html':        html,
        'from':        parseaddr(msgobj.get('From'))[1],
        'to':          parseaddr(msgobj.get('To'))[1],
        'cc':          parseaddr(msgobj.get('Cc'))[1],
        'reply_to':    parseaddr(msgobj.get('Reply-To'))[1],
        'attachments': attachments,
    }


class Imap():
    """
    Parse mails from imap server..
    """

    help = "Parse mails from settings.IMAP_SERVER"

    def __init__(self, *args, **kwargs):
        self.imap_server = kwargs.get('imap_server', None)
        self.imap_login = kwargs.get('imap_login', None)
        self.imap_passwd = kwargs.get('imap_passwd', None)
        self.parse_mail = kwargs.get('parse', self._get_parse)
        self.feedback = False

    def open(self):
        self.mailbox = IMAP4(self.imap_server)
        self.mailbox.login(self.imap_login, self.imap_passwd)
        self.mailbox.select()

    def _get_parse(self):
        def _parse(mail=None, *args, **kwargs):
            return mail

        return _parse

    def get_list(self):
        '''
        get list of message's numbers in main mailbox
        '''
        typ, data = self.mailbox.search(None, 'UNSEEN')
        nr_list = data[0].split(' ')
        if len(nr_list) == 1 and nr_list[0] == '':
            return []
        else:
            return nr_list

    def iterator(self):
        for number in self.get_list():
            typ, data = self.mailbox.fetch(number, '(RFC822 UID BODY[TEXT])')
            if data and len(data) > 1:
                mail = parse_email_content(email.message_from_string(data[0][1]))
                yield mail
            self.set_seen(number)
        rest = self.get_list()
        if len(rest) > 0:
            yield from self.iterator()

    def parse(self, _parse=None):
        '''
        parse all mails from imap mailbox
        '''
        return map(self.parse_mail, self.iterator())

    def set_seen(self, number):
        '''
        Set mail as seen
        '''
        self.mailbox.store(number, '+FLAGS', '\\Seen')
        self.mailbox.expunge()

    def mv_mail(self, number, mbox_name):
        '''
        move message to internal mailbox
        @param number: message number in main mailbox
        @param mbox_name: internal mailbox name
        '''
        self.mailbox.copy(number, "INBOX.%s" % mbox_name)
        self.mailbox.store(number, '+FLAGS', '\\Deleted')
        self.mailbox.expunge()

    def close(self):
        if self.mailbox:
            self.mailbox.close()
            self.mailbox.logout()

    def __del__(self, *args, **kwargs):
        self.close()



class MappingDict(dict):

    def __setitem__(self, key, item):
        self.__dict__[key] = item

    def __getitem__(self, key):
        return self.__dict__[key]

    def __repr__(self):
        return repr(self.__dict__)

    def __len__(self):
        return len(self.__dict__)

    def __delitem__(self, key):
        del self.__dict__[key]

    def clear(self):
        return self.__dict__.clear()

    def copy(self):
        return self.__dict__.copy()

    def has_key(self, k):
        return k in self.__dict__

    def update(self, *args, **kwargs):
        return self.__dict__.update(*args, **kwargs)

    def keys(self):
        return self.__dict__.keys()

    def values(self):
        return self.__dict__.values()

    def items(self):
        return self.__dict__.items()

    def pop(self, *args):
        return self.__dict__.pop(*args)

    def __cmp__(self, dict_):
        return self.__cmp__(self.__dict__, dict_)

    def __contains__(self, item):
        return item in self.__dict__

    def __iter__(self):
        return iter(self.__dict__)

    def __str__(self):
        return str(self.__dict__)


class MappingAttrs(MappingDict):

    def __getattr__(self, item):
        return self.__getitem__(item)

    def __setattr__(self, key, value):
        return self.__setitem__(key, value)

    def __delattr__(self, item):
        self.__delitem__(item)

class MappingDictDefault(MappingDict):

    DEFAULT_VALUE = None

    def __getitem__(self, key):
        return self.__dict__[key] if key in self.__dict__ else self.DEFAULT_VALUE


class MappingConst(MappingDictDefault):
    read_only_exception = PermissionError

    def __read_only(self, *args, **kwargs):
        raise self.read_only_exception

    def __init__(self):
        self.clear = self.__read_only
        self.update = self.__read_only
        self.pop = self.__read_only

        self.__delitem__ = self.__read_only
        self.__setitem__ = self.__read_only
        super().__init__()



import math
import json
import pprint

from .bereTools import *
from .config import get_config, EnvValue, join_path_value, join_path
from .imap import Imap
from .path import get_path_fun
from .patterns import match
from .singleton import Singleton
from .tail_recursion import tail_call_optimized
from .trace import info
from .trace import log_fun

SEND_TO_COUT = True
_pp = pprint.PrettyPrinter(indent=2)


def try_int(data, default=None):
    try:
        if isinstance(data, (list, tuple)):
            return try_int(data[0], default)
        return int(data)
    except Exception:
        return default


def get_pp(*args, **kwargs):
    __p = p(*args, **kwargs)

    def __pp(*datas):
        return __p(*datas)

    return __pp


def pp(txt, parse=None, cout=SEND_TO_COUT, string=None):
    if string is None:
        string = not cout
    if parse == 'json':
        _txt = json.loads(txt)
    elif parse:
        _txt = parse(txt)
    else:
        _txt = txt
    if cout:
        _pp.pprint(_txt)
    if string:
        return _pp.pformat(_txt)


def p(parse=None, cout=SEND_TO_COUT, **kwargs):
    def __pp(*txts):
        return [pp(txt=txt, parse=parse, cout=cout, **kwargs) for txt in txts]
    return __pp


def jprint(json_txt, cout=SEND_TO_COUT):
    return pp(txt=json_txt, parse='json', cout=cout)


def del_key(dict, key):
    if key in dict:
        del dict[key]
        return True
    return False


def none_on_error(fun):
    def __wrap_fun(*args, **kwargs):
        try:
            return fun(*args, **kwargs)
        except Exception as e:
            fun.error = e
            return None

    return __wrap_fun


def isNan(val):
    try:
        return math.isnan(val)
    except TypeError:
        return False


def isInf(val):
    try:
        return math.isinf(val)
    except TypeError:
        return False


def parse_float(val):
    val = float(val)

    if isNan(val) or isInf(val):
        return None

    return val



# -*- coding: utf-8 -*-
import os
import pprint

__author__ = 'beret'

try:
    from django.conf import settings

    BASE_DIR = settings.BASE_DIR
    DEBUG_FILE = getattr(settings, 'DEBUG_FILE', os.path.join(BASE_DIR, '_debug_log.log'))
except ImportError:
    BASE_DIR = os.path.join(os.path.abspath(__file__), '..')
    DEBUG_FILE = os.path.join(BASE_DIR, '_debug_log.log')

_cout = True
_pp = pprint.PrettyPrinter(indent=2)


def _get_write(f):
    if f:
        _debug_file = open(DEBUG_FILE, 'a') if f == True else f
        return _debug_file.write
    return False


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def env(fun):
    def _fun_wrap(*args, **kwargs):
        import os
        import sys

        os.environ.setdefault("DJANGO_SETTINGS_MODULE", "coderside.settings")
        sys.path.append(BASE_DIR)

        return fun(*args, **kwargs)

    return _fun_wrap



import math
from random import choice
from typing import Sequence, Optional
from dataclasses import dataclass


@dataclass
class Codes:

    DEFAULT_CHARS = '23456789abcdefghijkmnrstuvwxyz'
    chars: Sequence = DEFAULT_CHARS
    count: Optional[int] = None

    @property
    def chars_len(self):
        return len(self.chars)

    def __call__(
            self,
            length: Optional[int] = None,
            count: Optional[int] = None
    ):
        if length is None:
            if count is None:
                count = self.count
            length = self.code_len(count)
        return "".join([choice(self.chars) for _ in range(length)])

    def code_len(self, count):
        length = math.log(count, self.chars_len)
        return int(length) + 1


def get_code(*args, **kwargs):
    if "chars" in kwargs:
        chars = kwargs.pop("chars")
    else:
        chars = Codes.DEFAULT_CHARS
    codes_generator = Codes(chars=chars)
    return codes_generator(*args, **kwargs)



import os
import time

try:
    import psycopg2
except ImportError:
    raise ImportError("No psycopg2 module")

NAME = os.environ.get('POSTGRES_DB', 'test_db')
USER = os.environ.get('POSTGRES_USER', 'postgres')
PASSWORD = os.environ.get('POSTGRES_PASSWORD', 'postgres')
HOST = os.environ.get('POSTGRES_HOST', 'localhost')
PORT = os.environ.get('POSTGRES_PORT', 5432)
RETRIES = os.environ.get('POSTGRES_RETRIES', 120)
WAIT = os.environ.get('POSTGRES_WAIT', 5)

for i in range(RETRIES):
    try:
        with psycopg2.connect(
                user=USER,
                password=PASSWORD,
                host=HOST,
                port=PORT,
                database=NAME
        ):
            print("PostgreSQL is ready for connection now!")
            break

    except (Exception, psycopg2.Error) as error:
        print("Error while connecting to PostgreSQL", error)
        print("Let wait {} seconds for postgresql connection {} time".format(WAIT, i + 1))
        time.sleep(WAIT)



from .tail_recursion import tail_call_optimized


def rest(l):
    if len(l) > 1:
        return l[1:]
    else:
        return None


def raw_match(pattern, _str, star=False, at=False):
    if pattern is None:
        if _str is None or star:
            return True
        elif at:
            return len(_str) == 1
        else:
            return False

    if pattern[0] == '*':
        star = True
        return match(rest(pattern), _str, star)

    if pattern[0] == '@':
        if star:
            return match(rest(pattern), _str, star)
        else:
            at = True
            return match(rest(pattern), _str, star, at)

    if _str is None:
        return False

    if star:
        if match(pattern, _str, star=False, at=False):
            return True
        else:
            return match(pattern, rest(_str), star, at)

    if at:
        if match(pattern, _str, star=False, at=False):
            return True
        else:
            return match(pattern, rest(_str), star=False, at=False)

    if pattern[0] in ['?', _str[0]]:
        return match(rest(pattern), rest(_str), star, at)

    return False


def match(pattern, _str, star=False, at=False):
    return tail_call_optimized(raw_match)(pattern, _str, star, at)



__author__ = 'beret'

from threading import local

# to fix Django >= 1.10
try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object

import urllib

_local = local()


class GlobalRequestMiddleware(MiddlewareMixin):
    def process_request(self, request):
        _local.request = request


def get_current_request():
    return getattr(_local, "request", None)


def get_current_user():
    request = get_current_request()
    if request and request.user:
        return request.user


def get_current_state():
    request = get_current_request()
    return urllib.urlencode(request.GET)


def get_client_ip(request=None):
    if not request:
        request = get_current_request()
    remote_address = request.META.get('REMOTE_ADDR')
    # set the default value of the ip to be the REMOTE_ADDR if available
    # else None
    ip = remote_address
    # try to get the first non-proxy ip (not a private ip) from the
    # HTTP_X_FORWARDED_FOR
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        proxies = x_forwarded_for.split(',')
        PRIVATE_IPS_PREFIX = ('10.', '172.', '192.', '127.')
        # remove the private ips from the beginning
        while (len(proxies) > 0 and
               proxies[0].startswith(PRIVATE_IPS_PREFIX)):
            proxies.pop(0)
        # take the first ip which is not a private one (of a proxy)
        if len(proxies) > 0:
            ip = proxies[0]

    return ip



#!/usr/bin/env python
# -*- coding: utf-8 -*-

class Table:
    """
    class for manual SQL table manipulation
    """

    def __init__(self, db, name):
        """class initialise
        arguments:
            db:     data base object
            name:   name of destination table
        modifies fields:
            fields: table's column list
            name:   name of destination table
            db:     database object
            id:     name of destination table's primary key
        """
        self.name = name
        self.db = db
        cl = self.db.cursor()
        sql = "describe `%s`;" % str(name)
        cl.execute(sql)
        self.fields = {}
        fields = cl.fetchall()
        for field in fields:
            if 'auto_increment' in field[-1]:
                self.id = field[0]
            self.fields[field[0]] = field

    def getValues(self, table_name, col_name=False, col_val=False, sql=False):
        """take data from another table
        arguments:
            table_name: source table name
            col_name:   name of column to filter records
            col_val:    filtered value
            sql:        custom query
        modifies fields:
            values:     data to insert to destination table
            columns:    column's names in destination table
        """
        if not sql:
            sql = "select * from `%s` "
        if table_name:
            sql = sql % str(table_name)
        if col_name and col_val:
            sql = sql + " where %s in (" % col_name
            a = ''
            for x in col_val:
                sql = sql + a + "'%s'" % str(x)
                a = ','
            sql = sql + ")"
        sql = sql + ";"
        #        print sql

        cl = self.db.cursor()
        cl.execute(sql)
        self.values = cl.fetchall()
        self.columns = map(lambda x: x[0], cl.description)

    def transValues(self, column, trans_dict):
        """
        translate value of given column by given dictionary
        arguments:
            column:        name of column to translation
            trans_dict:    translation dictionary
        modifies fields:
            values:        in given column change values by translation dictionary
        """
        new_values = []
        id_index = self.columns.index(self.id)
        trans_index = self.columns.index(column)
        self.fields = self.__excludeId(self.columns, id_index)
        for v in self.values:
            value = list(v)
            value[trans_index] = trans_dict[value[trans_index]]
            new_values.append(value)
        self.values = new_values

    def sqlInsert(self, table, values, fields=False):
        """create SQL query
        arguments:
            table:    table name
            values:   values
            fields:   field's names to insert values
        """
        sql = "insert into `%s` " % table
        if fields:
            sql = sql + "("
            for field in fields:
                sql = sql + "`%s`," % str(field)
            sql = sql[:-1] + ") "
        sql = sql + "values ("
        for value in values:
            sql = sql + "%s," % self.sqlValue(value)
        sql = sql[:-1]
        sql = sql + ");\n"
        return sql

    def sqlValue(self, value):
        """
        change value to SQL text format
        """
        if type(value) == type(None):
            return "NULL"
        else:
            return "'%s'" % self.db.escape_string(str(value))

    def insertValues(self, column=False, trans_dict=False):
        """
        insert new values to destination table
        arguments:
            column:        foregin key column to translation
            trans_dict:    translation dictionary
        """
        cl = self.db.cursor()
        self.ids = {}
        self.bads = []
        id_index = self.columns.index(self.id)
        if column:
            self.transValues(column, trans_dict)
        self.fields = self.__excludeId(self.columns, id_index)
        for value in self.values:
            sql = self.sqlInsert(self.name, self.__excludeId(value, id_index), self.fields)
            if cl.execute(sql):
                self.ids[value[id_index]] = cl.lastrowid
            else:
                self.bads.append(value)
                print(value)

    def __excludeId(self, old, index):
        new = list(old)
        new.pop(index)
        return new


def insert_value(table, values, fields=False):
    """generate SQL query to insert given values
    parametry:
        table:    table name
        values:   values to insert
        fields:   optionaly field's names
    """
    sql = "insert into `%s` " % table
    if fields:
        sql = sql + "("
        for field in fields:
            sql = sql + "`%s`," % str(field)
        sql = sql[:-1] + ") "
    sql = sql + "values ("
    for value in values:
        if type(value) == type(None):
            sql = sql + " NULL , "
        else:
            sql = sql + "'%s'," % str(value)
    sql = sql[:-1]
    sql = sql + ");\n"
    return sql



import os
import fnmatch


def get_path_fun(depth=0):
    _dir = os.getcwd()
    while depth:
        _dir = os.path.dirname(_dir)
        depth -= 1
    BASE_DIR = _dir

    def __get_path(*paths):
        file_name = None
        if len(paths) == 1:
            if isinstance(paths[0], str):
                "if is one argument, and it is a string, get it as file_name"
                file_name = paths[0]
            else:
                "if is one argument, and it is not a string, get it as argument list to join"
                paths = paths[0]

        if file_name is None:
            file_name = os.path.join(*paths)

        file_name = os.path.expandvars(file_name)
        if file_name[0] != '/':
            return os.path.join(BASE_DIR, file_name)
        return os.path.abspath(file_name)

    return __get_path


def all_files(root, patterns="*", single_level=False, yield_folders=False):
    """return all files in given directory"""
    patterns = patterns.split(';')
    for path, subdirs, files in os.walk(root):
        if yield_folders:
            files.extend(subdirs)
        files.sort()
        for name in files:
            for pattern in patterns:
                if fnmatch.fnmatch(name, pattern):
                    yield os.path.join(path, name)
                    break
        if single_level:
            break



import pprint
import sys

_pp = pprint.PrettyPrinter(indent=2)

LOG_EVENTS = ('return', 'call')


def log_fun(*args, **kwargs):
    """
    decorator
    log call, arguments, local variables values and returned value
    :param call: log call
    :param ret:  log return
    :param content: log all local variables value at return
    :param path: log call and return point path and line number
    :param args: log args
    :param log_fun: custom log function, default is print
    """
    show_call = kwargs.get("call", True)
    show_return = kwargs.get("ret", True)
    show_content = kwargs.get("content", True)
    show_path = kwargs.get("path", True)
    show_args = kwargs.get("args", True)
    log_fun = kwargs.get("log_fun", print)

    def log_content(fun):

        def print_log(frame, event):
            co = frame.f_code
            line_no = frame.f_lineno
            filename = co.co_filename
            path = "{}:{}".format(filename, line_no) if show_path else ""
            variables = ["%s=%s" % (key, _pp.pformat(val)) for key, val in frame.f_locals.items()]

            args = "({})".format(",".join(variables)) if show_args else ""
            if event == 'call':
                if show_call:
                    func_name = co.co_name
                    log_fun("{}{}{}".format("%s " % path, func_name, args))
            elif show_content:
                log_fun("\n".join(variables))
                if show_path:
                    log_fun(path)

        def log_return(old_log=None):
            def _log_return_wrapper(frame, event, arg):
                if frame.f_code == fun.__code__ and event in LOG_EVENTS:
                    print_log(frame, event)
                if callable(old_log):
                    old_log(frame, event, arg)

                return _log_return_wrapper

            return _log_return_wrapper

        def _wrapper(*args, **kwargs):
            old_log = sys.gettrace()
            sys.settrace(log_return(old_log))
            try:
                result = fun(*args, **kwargs)
            except:
                raise
            else:
                if (show_return):
                    log_fun("return {}\n".format(result))
                return result
            finally:
                sys.settrace(old_log)

        return _wrapper

    if args and hasattr(args[0], "__call__"):
        return log_content(args[0])

    return log_content


def info(object, spacing=10, collapse=1):
    "print any object's method and their docs"
    methodList = [method for method in dir(object) if callable(getattr(object, method))]
    processFunc = collapse and (lambda s: " ".join(s.split())) or (lambda s: s)
    print(
        "\n".join(["%s %s" %
                   (method.ljust(spacing),
                    processFunc(str(getattr(object, method).__doc__)))
                   for method in methodList])
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import urllib

# fixed point Y combinator
Y = (lambda g: (lambda f: g(lambda *arg: f(f)(*arg)))(lambda f: g(lambda *arg: f(f)(*arg))))
# example of using Y combinator to calculate greatest common divisor by euliclean algoritm
# gcd=lambda a,b: a if not b else nwd(b,a%b)
gcd = lambda a, b: Y(lambda f: lambda a, b: not b and a or f(b, a % b))(a, b)
# function for division of rational numbers
div = lambda a, b: (int(a) / int(b), (a % b) / gcd((a % b), b), b / gcd((a % b), b))

anonim_div = lambda a, b: (int(a) / int(b), (a % b) / (
    lambda a, b: (lambda g: (lambda f: g(lambda *arg: f(f)(*arg)))(lambda f: g(lambda *arg: f(f)(*arg))))(
        lambda f: lambda a, b: not b and a or f(b, a % b))(a, b))((a % b), b), b / (lambda a, b: (
    lambda g: (lambda f: g(lambda *arg: f(f)(*arg)))(lambda f: g(lambda *arg: f(f)(*arg))))(
    lambda f: lambda a, b: not b and a or f(b, a % b))(a, b))((a % b), b))

"""
encode and decode given number to text value
"""
A = "23456789abcdefghijkmnpqrstuvwxyzABCDEFGHJKLMNPQRSTUVWXYZ"
_be = lambda x, ret: _be(x // len(A), "%s%s" % (ret, A[x % len(A)])) if x >= len(A) else "%s%s" % (ret, A[x])
base_encoding = lambda x: Y(
    lambda f: lambda x, ret: f(x // len(A), "%s%s" % (ret, A[x % len(A)])) if x >= len(A) else "%s%s" % (ret, A[x]))(x,
                                                                                                                     "")
_bd = lambda x, ret: _bd(x[:-1], len(A) * ret + A.index(x[-1])) if len(x) else ret
base_decoding = lambda x: Y(lambda f: lambda x, ret: f(x[:-1], len(A) * ret + A.index(x[-1])) if len(x) else ret)(x, 0)

default_if_null = lambda obj, name, default: (lambda x, y: x if x else y)(getattr(obj, name, default), default)


def makeFun(data):
    """
    check with the argument's data is a function, else generate function for return given data
    """
    if callable(data):
        return data
    else:
        return lambda x: data


def makeFindFun(data):
    """
    check witch the argument's data is a function, else generate function for finding in their argument a given data
    """
    if callable(data):
        return data
    else:
        return lambda x, i: (x.find(data, i), len(data))


def replace(beg, end, new):
    """
    generate function for replacing any data in given text
    * beg - string of begin area to replace or function who given data and index and return new index, after given to begin area to replace
    * end - string of end area to replace or function who given data and index and return new index, after given to end area to replace
    * new - string to replace instead of finding area or function who given area's data and return a new value for this place
    """
    beg, end = map(makeFindFun, (beg, end))
    new = makeFun(new)

    def func(txt):
        e = 0
        while (True):
            le = e
            b, l = beg(txt, e)
            if b == -1:
                break
            b = b + l
            e, l = end(txt, b)
            if e == -1:
                break
            yield "%s%s" % (txt[le:b], new(txt[b:e]))
        yield txt[le:]

    return func


def addLine(msg, line, sep='\n'):
    """
    just add new line to text variable
    """
    msg = "%s%s" % (msg, sep) if msg else ""
    msg = "%s%s" % (msg, line)
    return msg


def today():
    """
    just take date
    """
    from datetime import date
    t = date.today()
    return (t.day, t.month, t.year)


def getchar():
    '''
    is a UNIX function replacement, working on true UNIX console only (not under any windows game systems like Vista, XP or 7)
    '''
    try:
        import termios

        fd = sys.stdin.fileno()

        if os.isatty(fd):

            old = termios.tcgetattr(fd)
            new = termios.tcgetattr(fd)
            new[3] = new[3] & ~termios.ICANON & ~termios.ECHO
            new[6][termios.VMIN] = 1
            new[6][termios.VTIME] = 0

            try:
                termios.tcsetattr(fd, termios.TCSANOW, new)
                termios.tcsendbreak(fd, 0)
                ch = os.read(fd, 7)

            finally:
                termios.tcsetattr(fd, termios.TCSAFLUSH, old)
        else:
            ch = os.read(fd, 7)

    except IOError:
        ch = input()
    return ch


def url2txt(url, coding=False):
    """
    simple return text data from given url
    """

    try:
        request = urllib.Request(url)
        response = urllib.urlopen(request)
        #        if coding=='auto':
        #            re_coding=re.compile(r'encoding="Windows-1250"')
        if coding:
            return response.read().decode(coding)
        else:
            return response.read()
    except Exception:
        return False


def tags(tag, text, lista=False):
    """
    return given sgml like tag's values in given text
    """
    tag = tag.lower()
    if not lista:
        lista = []
    if "<%s" % tag not in text.lower():
        return lista
    else:
        row_begin = text.lower().find("<%s" % tag)
        row_begin = text.lower().find('>', row_begin) + 1
        row_end = text.lower().find("</%s" % tag, row_begin)
        lista.append(text[row_begin:row_end])
        return tags(tag, text[text.lower().find('>', row_end):], lista)


def isFloat(s):
    try:
        float(s)
    except (ValueError, TypeError):
        return False
    else:
        return True


def toType(s, to_type=int):
    try:
        x = to_type(s)
    except (ValueError, TypeError):
        return x
    else:
        return True


import os

from abc import abstractmethod
from typing import Optional, Iterable, Type, AnyStr

from .mapping import MappingConst


class Value:

    def __call__(self, *envs: dict) -> Optional[AnyStr]:
        for env in envs:
            result = self.get_value(env)
            if result is not None:
                return result
        return None

    @abstractmethod
    def get_value(self, config: object) -> AnyStr:
        pass


class EnvValue(Value):
    def __init__(self, var_name: str):
        self.var_name = var_name

    def get_value(self, env):
        return env.get(self.var_name)


def join_path(*args):
    return os.path.join(*args)


def join_path_value(dir_id):
    """
    produce a special parser with join another variable value to given path
    """
    class JoinPathValue(EnvValue):
        def get_value(self, env):
            if dir_id in env:
                value = join_path(env[dir_id], self.var_name)
                return value
    return JoinPathValue


def bool_value(value):
    try:
        i = float(value)
        return bool(i)
    except ValueError:
        pass

    s = str(value).lower()
    if "true" in s:
        return True
    if "false" in s:
        return False
    return None


def expand_defaults(init):
    return (
        map(
            lambda args:
            (lambda key, value=None, parser=str, env_key=None, parse_default=True:
             (
                 key,
                 value,
                 parser,
                 key if env_key is None else env_key,
                 parse_default
             )
             )(*args),
            init
        )
    )


class Config(MappingConst):

    @abstractmethod
    def get_values(self) -> dict:
        return {}

    def __init__(self):
        super().__init__()
        self.__dict__ = self.get_values()


def init_envs(envs: Optional[Iterable]) -> list:
    return list(envs) if envs is not None else []


class ConfigEnv(Config):
    DEFAULTS = []

    def get_envs(self, envs: Optional[Iterable[dict]] = None) -> list:
        envs = init_envs(envs)
        envs.append(os.environ)
        return envs

    def get_values(self, init=None):
        if init is not None:
            defaults = expand_defaults(init)
        else:
            defaults = self.DEFAULTS
        values = {}
        envs = self.get_envs()
        for key, value, parser, env_key, parse_default in defaults:
            env_val = EnvValue(env_key)(*envs)

            if env_val is not None:
                value = parser(env_val)
            elif parse_default and not callable(value):
                value = parser(value)

            while callable(value):
                value = value(values, *envs)
                if parser:
                    value = parser(value)
                    parser = False

            values[key] = value
        return values

    def update(self, init):
        return super().update(self.get_values(init))


class ConfigEnvFiles(ConfigEnv):
    ENV_FILES = []

    def get_env_files(self) -> dict:
        env = {}
        for env_file_path in self.ENV_FILES:
            try:
                with open(env_file_path, 'r') as env_file:
                    for line in env_file.readlines():
                        line = line.strip()
                        if '#' in line:
                            line = line.split('#')[0].strip()
                        if '=' in line:
                            try:
                                key, value = line.split('=')
                                env[key.strip()] = value.strip()
                            except ValueError:
                                pass
            except FileNotFoundError:
                pass
        return env

    def get_envs(self, envs: Optional[Iterable[dict]] = None) -> list:
        envs = super().get_envs(envs)
        envs.append(self.get_env_files())
        return envs


def get_config(
        defaults: Iterable[tuple],
        env_files: Optional[Iterable[AnyStr]] = None,
        config_class: Optional[Type[Config]] = ConfigEnvFiles
) -> Type[Config]:

    class ConfigClass(config_class):
        DEFAULTS = expand_defaults(defaults)
        ENV_FILES = env_files or []

    return ConfigClass



'''
Created on 2010-08-20

@author: beret
'''

# !/usr/bin/env python

import imaplib
import sys


def cpimap(user1, host1, pass1, user2, host2, pass2, target='/'):
    m1 = imaplib.IMAP4_SSL(host1)
    m2 = imaplib.IMAP4_SSL(host2)
    m1.login(user1, pass1)
    m2.login(user2, pass2)

    folders = [folder.split(' "/" ')[1][1:-1] for folder in filter(lambda x: x, m1.list()[1])]
    folders2 = [folder.split(' "/" ')[1][1:-1] for folder in filter(lambda x: x, m2.list()[1])]
    #    folders.insert( 0, target ) # Copy messages in the root of the target too

    print('Copying', len(folders), 'folders:')
    for f in folders:
        print(f)

    for f in folders:
        if '\\' in f:
            print('Skipping', f)
            # imaplib does not support backslashes in mailbox names!
            continue
        print('Copying', f)
        if f not in folders2:
            m2.create(f)
        m1.select(f)
        print('Fetching messages...')
        typ, data = m1.search(None, 'ALL')

        msgs = data[0].split()

        sys.stdout.write(" ".join(['Copying', str(len(msgs)), 'messages']))

        for num in msgs:
            typ, data = m1.fetch(num, '(RFC822)')
            sys.stdout.write('.')
            m2.append(f, None, None, data[0][1])
        sys.stdout.write('\n')


if __name__ == '__main__':
    user1 = 'test'
    host1 = 'test.ceenet.org'
    pass1 = 'test'
    user2 = 'test@hipisi.org.pl'
    host2 = 'imap.gmail.com'
    pass2 = 'test'
    cpimap(user1, host1, pass1, user2, host2, pass2)



#!/usr/bin/env python
import os
import subprocess
import sys
from xml.dom.minidom import parseString


def getXmlData(cmd):
    xml_diff = subprocess.getoutput(cmd)
    xml_diff = xml_diff[xml_diff.find("<?"):]
    return parseString(xml_diff)


def getCmd(src, rev_begin, rev_end):
    if src[:7] == 'http://':
        url = src
        cmd = "svn diff --summarize --xml -r %(rbegin)s:%(rend)s %(url)s" % {'url':  url, 'rbegin': str(rev_begin),
                                                                             'rend': str(rev_end)
        }
    else:
        dir = os.path.abspath(src)
        if not os.path.isdir(dir):
            dir = os.path.dirname(dir)
        cmd = "svn diff --summarize --xml -r %(rbegin)s:%(rend)s %(dir)s" % {'dir':  dir, 'rbegin': str(rev_begin),
                                                                             'rend': str(rev_end)
        }
    return cmd


def getFiles(xml_data):
    files = []
    nodes = xml_data.getElementsByTagName("path")
    for node in nodes:
        if node.attributes['item'].value != "deleted":
            files.append(node.childNodes[0].data)
    return files


def crateDestByDir(dir, dest, files):
    dest = os.path.abspath(dest)
    files = map(lambda file: os.path.abspath(file)[len(dir):], files)
    for file in files:
        src_file = os.path.abspath(dir + os.path.sep + file)
        desc_file = os.path.abspath(dest + os.path.sep + file)
        desc_dir = os.path.dirname(desc_file)
        if not os.path.exists(desc_dir):
            os.makedirs(desc_dir)
        cmd = "cp -vf %s %s" % (src_file, desc_file)
        print(subprocess.getoutput(cmd))


def crateDestByUrl(url, dest, rev_end, urls):
    dest = os.path.abspath(dest)
    for src_url in urls:
        src_file = src_url[len(url):]
        desc_file = os.path.abspath(dest + os.path.sep + src_file)
        desc_dir = os.path.dirname(desc_file)
        if not os.path.exists(desc_dir):
            os.makedirs(desc_dir)
        cmd = "svn export --force -q -r %(rend)s %(src_url)s %(desc_file)s" % {'src_url':   src_url,
                                                                               'desc_file': desc_file, 'rend': rev_end
        }
        out = subprocess.getoutput(cmd)
        if out:
            print(out)
        print("%s -> %s" % (src_url, desc_file))


if __name__ == '__main__':
    argv = sys.argv
    if argv[1][:7] == 'http://':
        url = argv[1]
        argv.remove(url)
    else:
        url = False
    dest = argv[1]
    rev_begin = argv[2]
    rev_end = argv[3] if len(argv) > 3 else 'HEAD'
    if len(argv) > 4:
        dir = argv[4]
    elif not url:
        dir = os.path.abspath('.')
    else:
        dir = False

    src = url if url else dir
    cmd = getCmd(src, rev_begin, rev_end)

    print(cmd)

    data = getXmlData(cmd)
    files = getFiles(data)
    if dir:
        crateDestByDir(dir, dest, files)
    elif url:
        crateDestByUrl(url, dest, rev_end, files)
